@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.8.3+   2012/05/05
+ * Version: 1.8.3+   2015/04/08
  */
 
 #ifndef _SECURITY_CCSECURITY_INTERNAL_H
@@ -56,6 +56,9 @@
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 6)
 #include <linux/kthread.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
+#include <linux/magic.h>
+#endif
 #include <stdarg.h>
 #include <asm/uaccess.h>
 #include <net/sock.h>
@@ -178,6 +181,10 @@ struct path {
 	})
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0)
+#define f_vfsmnt f_path.mnt
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
 
 /**
@@ -353,6 +360,10 @@ static inline void __list_del_entry(struct list_head *entry)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 30) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9))
 
+#if LINUX_VERSION_CODE == KERNEL_VERSION(2, 4, 21)
+#undef ssleep
+#endif
+
 #ifndef ssleep
 
 /**
@@ -1583,7 +1594,7 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 bool ccs_memory_ok(const void *ptr, const unsigned int size);
 char *ccs_encode(const char *str);
 char *ccs_encode2(const char *str, int str_len);
-char *ccs_realpath(struct path *path);
+char *ccs_realpath(const struct path *path);
 const char *ccs_get_exe(void);
 const struct ccs_path_info *ccs_get_name(const char *name);
 int ccs_audit_log(struct ccs_request_info *r);
@@ -1976,6 +1987,14 @@ bool ccs_used_by_cred(const struct ccs_domain_info *domain);
 int ccs_start_execve(struct linux_binprm *bprm, struct ccs_execve **eep);
 void ccs_finish_execve(int retval, struct ccs_execve *ee);
 void ccs_load_policy(const char *filename);
+#ifndef CONFIG_AKARI_TRACE_EXECVE_COUNT
+#define ccs_audit_alloc_execve(ee) do { } while (0)
+#define ccs_audit_free_execve(ee, is_current)	do { } while (0)
+#else
+void ccs_audit_alloc_execve(const struct ccs_execve * const ee);
+void ccs_audit_free_execve(const struct ccs_execve * const ee,
+			   const bool is_current);
+#endif
 
 #define CCS_TASK_SECURITY_HASH_BITS 12
 #define CCS_MAX_TASK_SECURITY_HASH (1u << CCS_TASK_SECURITY_HASH_BITS)
