@@ -1,14 +1,13 @@
 /*
  * lsm.c
  *
- * Copyright (C) 2010-2012  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
+ * Copyright (C) 2010-2013  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 1.0.27   2012/05/05
+ * Version: 1.0.32   2015/04/08
  */
 
 #include "internal.h"
-#include <linux/security.h>
-#include <linux/namei.h>
+#include "probe.h"
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0)
 #define USE_UMODE_T
 #else
@@ -57,6 +56,8 @@ static struct ccs_security ccs_default_security = {
 extern struct ccs_security ccs_oom_security;
 /* Dummy security context for avoiding NULL pointer dereference. */
 extern struct ccs_security ccs_default_security;
+/* Dummy marker for calling security_bprm_free(). */
+static const unsigned long ccs_bprm_security;
 #endif
 
 /* For exporting variables and functions. */
@@ -82,6 +83,82 @@ static struct security_operations original_security_ops /* = *security_ops; */;
 #define CCS_INODE_HOOK_HAS_MNT
 #endif
 
+#ifdef CONFIG_AKARI_TRACE_EXECVE_COUNT
+
+/**
+ * ccs_update_ee_counter - Update "struct ccs_execve" counter.
+ *
+ * @count: Count to increment or decrement.
+ *
+ * Returns updated counter.
+ */
+static unsigned int ccs_update_ee_counter(int count)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 10) || defined(atomic_add_return)
+	/* Debug counter for detecting "struct ccs_execve" memory leak. */
+	static atomic_t ccs_ee_counter = ATOMIC_INIT(0);
+	return atomic_add_return(count, &ccs_ee_counter);
+#else
+	static DEFINE_SPINLOCK(ccs_ee_lock);
+	static unsigned int ccs_ee_counter;
+	unsigned long flags;
+	spin_lock_irqsave(&ccs_ee_lock, flags);
+	ccs_ee_counter += count;
+	count = ccs_ee_counter;
+	spin_unlock_irqrestore(&ccs_ee_lock, flags);
+	return count;
+#endif
+}
+
+/**
+ * ccs_audit_alloc_execve - Audit allocation of "struct ccs_execve".
+ *
+ * @ee: Pointer to "struct ccs_execve".
+ *
+ * Returns nothing.
+ */
+void ccs_audit_alloc_execve(const struct ccs_execve * const ee)
+{
+	printk(KERN_INFO "AKARI: Allocated %p by pid=%u (count=%u)\n", ee,
+	       current->pid, ccs_update_ee_counter(1) - 1);
+}
+
+/**
+ * ccs_audit_free_execve - Audit release of "struct ccs_execve".
+ *
+ * @ee:   Pointer to "struct ccs_execve".
+ * @task: True if released by current task, false otherwise.
+ *
+ * Returns nothing.
+ */
+void ccs_audit_free_execve(const struct ccs_execve * const ee,
+			   const bool is_current)
+{
+	const unsigned int tmp = ccs_update_ee_counter(-1);
+	if (is_current)
+		printk(KERN_INFO "AKARI: Releasing %p by pid=%u (count=%u)\n",
+		       ee, current->pid, tmp);
+	else
+		printk(KERN_INFO "AKARI: Releasing %p by kernel (count=%u)\n",
+		       ee, tmp);
+}
+
+#endif
+
+#if !defined(CONFIG_AKARI_DEBUG)
+#define ccs_debug_trace(pos) do { } while (0)
+#else
+#define ccs_debug_trace(pos)						\
+	do {								\
+		static bool done;					\
+		if (!done) {						\
+			printk(KERN_INFO				\
+			       "AKARI: Debug trace: " pos " of 4\n");	\
+			done = true;					\
+		}							\
+	} while (0)						 
+#endif
+
 /**
  * ccs_clear_execve - Release memory used by do_execve().
  *
@@ -100,12 +177,14 @@ static void ccs_clear_execve(int ret, struct ccs_security *security)
 	if (!ee)
 		return;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 	/*
 	 * Drop refcount on "struct cred" in "struct linux_binprm" and forget
 	 * it.
 	 */
 	put_cred(security->cred);
 	security->cred = NULL;
+#endif
 	atomic_dec(&ccs_in_execve_tasks);
 #endif
 	ccs_finish_execve(ret, ee);
@@ -124,6 +203,18 @@ static void ccs_rcu_free(struct rcu_head *rcu)
 {
 	struct ccs_security *ptr = container_of(rcu, typeof(*ptr), rcu);
 	struct ccs_execve *ee = ptr->ee;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+	/*
+	 * If this security context was associated with "struct pid" and
+	 * ptr->ccs_flags has CCS_TASK_IS_IN_EXECVE set, it indicates that a
+	 * "struct task_struct" associated with this security context exited
+	 * immediately after do_execve() has failed.
+	 */
+	if (ptr->pid && (ptr->ccs_flags & CCS_TASK_IS_IN_EXECVE)) {
+		ccs_debug_trace("1");
+		atomic_dec(&ccs_in_execve_tasks);
+	}
+#else
 	/*
 	 * If this security context was associated with "struct pid" and
 	 * remembers "struct cred" in "struct linux_binprm", it indicates that
@@ -131,46 +222,22 @@ static void ccs_rcu_free(struct rcu_head *rcu)
 	 * immediately after do_execve() has failed.
 	 */
 	if (ptr->pid && ptr->cred) {
-#ifdef CONFIG_AKARI_DEBUG
-		static bool done;
-		if (!done) {
-			printk(KERN_INFO "AKARI: Dropping refcount on "
-			       "\"struct cred\" in \"struct linux_binprm\" "
-			       "because some \"struct task_struct\" has "
-			       "exit()ed immediately after do_execve() has "
-			       "failed.\n");
-			done = true;
-		}
-#endif
+		ccs_debug_trace("1");
 		put_cred(ptr->cred);
 		atomic_dec(&ccs_in_execve_tasks);
 	}
+#endif
 	/*
 	 * If this security context was associated with "struct pid",
 	 * drop refcount obtained by get_pid() in ccs_find_task_security().
 	 */
 	if (ptr->pid) {
-#ifdef CONFIG_AKARI_DEBUG
-		static bool done;
-		if (!done) {
-			printk(KERN_INFO "AKARI: Dropping refcount on "
-			       "\"struct pid\".\n");
-			done = true;
-		}
-#endif
+		ccs_debug_trace("2");
 		put_pid(ptr->pid);
 	}
 	if (ee) {
-#ifdef CONFIG_AKARI_DEBUG
-		static bool done;
-		if (!done) {
-			printk(KERN_INFO "AKARI: Releasing memory in "
-			       "\"struct ccs_execve\" because some "
-			       "\"struct task_struct\" has exit()ed "
-			       "immediately after do_execve() has failed.\n");
-			done = true;
-		}
-#endif
+		ccs_debug_trace("3");
+		ccs_audit_free_execve(ee, false);
 		kfree(ee->handler_path);
 		kfree(ee);
 	}
@@ -258,7 +325,17 @@ static int ccs_task_create(unsigned long clone_flags)
 static int ccs_cred_prepare(struct cred *new, const struct cred *old,
 			    gfp_t gfp)
 {
-	int rc = ccs_copy_cred_security(new, old, gfp);
+	int rc;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+	/*
+	 * For checking whether reverting domain transition is needed or not.
+	 *
+	 * See ccs_find_task_security() for reason.
+	 */
+	if (gfp == GFP_KERNEL)
+		ccs_find_task_security(current);
+#endif
+	rc = ccs_copy_cred_security(new, old, gfp);
 	if (rc)
 		return rc;
 	if (gfp == GFP_KERNEL)
@@ -387,6 +464,32 @@ static void ccs_task_free_security(struct task_struct *p)
 	ccs_free_task_security(p);
 }
 
+/**
+ * ccs_bprm_alloc_security - Allocate memory for "struct linux_binprm".
+ *
+ * @bprm: Pointer to "struct linux_binprm".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_bprm_alloc_security(struct linux_binprm *bprm)
+{
+	int rc;
+	while (!original_security_ops.bprm_alloc_security);
+	rc = original_security_ops.bprm_alloc_security(bprm);
+	if (bprm->security || rc)
+		return rc;
+	/*
+	 * Update bprm->security to &ccs_bprm_security so that
+	 * security_bprm_free() is called even if do_execve() failed at
+	 * search_binary_handler() without allocating memory at
+	 * security_bprm_alloc(). This trick assumes that active LSM module
+	 * does not access bprm->security if that module did not allocate
+	 * memory at security_bprm_alloc().
+	 */
+	bprm->security = (void *) &ccs_bprm_security;
+	return 0;
+}
+
 /**
  * ccs_bprm_free_security - Release memory for "struct linux_binprm".
  *
@@ -396,8 +499,25 @@ static void ccs_task_free_security(struct task_struct *p)
  */
 static void ccs_bprm_free_security(struct linux_binprm *bprm)
 {
-	while (!original_security_ops.bprm_free_security);
-	original_security_ops.bprm_free_security(bprm);
+	/*
+	 * If do_execve() succeeded, bprm->security will be updated to NULL at
+	 * security_bprm_compute_creds()/security_bprm_apply_creds() if
+	 * bprm->security was set to &ccs_bprm_security at
+	 * security_bprm_alloc().
+	 *
+	 * If do_execve() failed, bprm->security remains at &ccs_bprm_security
+	 * if bprm->security was set to &ccs_bprm_security at
+	 * security_bprm_alloc().
+	 *
+	 * And do_execve() does not call security_bprm_free() if do_execve()
+	 * failed and bprm->security == NULL. Therefore, do not call
+	 * original_security_ops.bprm_free_security() if bprm->security remains
+	 * at &ccs_bprm_security .
+	 */
+	if (bprm->security != &ccs_bprm_security) {
+		while (!original_security_ops.bprm_free_security);
+		original_security_ops.bprm_free_security(bprm);
+	}
 	/*
 	 * If do_execve() succeeded,
 	 * ccs_clear_execve(0, ccs_current_security());
@@ -420,6 +540,8 @@ static void ccs_bprm_free_security(struct linux_binprm *bprm)
  */
 static void ccs_bprm_compute_creds(struct linux_binprm *bprm)
 {
+	if (bprm->security == &ccs_bprm_security)
+		bprm->security = NULL;
 	while (!original_security_ops.bprm_compute_creds);
 	original_security_ops.bprm_compute_creds(bprm);
 	ccs_clear_execve(0, ccs_current_security());
@@ -437,6 +559,8 @@ static void ccs_bprm_compute_creds(struct linux_binprm *bprm)
  */
 static void ccs_bprm_apply_creds(struct linux_binprm *bprm, int unsafe)
 {
+	if (bprm->security == &ccs_bprm_security)
+		bprm->security = NULL;
 	while (!original_security_ops.bprm_apply_creds);
 	original_security_ops.bprm_apply_creds(bprm, unsafe);
 	ccs_clear_execve(0, ccs_current_security());
@@ -491,12 +615,14 @@ static int ccs_bprm_check_security(struct linux_binprm *bprm)
 		rc = ccs_start_execve(bprm, &security->ee);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 		if (security->ee) {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 			/*
 			 * Get refcount on "struct cred" in
 			 * "struct linux_binprm" and remember it.
 			 */
 			get_cred(bprm->cred);
 			security->cred = bprm->cred;
+#endif
 			atomic_inc(&ccs_in_execve_tasks);
 		}
 #endif
@@ -878,9 +1004,11 @@ static int ccs_inode_setattr(struct dentry *dentry, struct iattr *attr)
 #if !defined(CONFIG_SECURITY_PATH) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
 	if (attr->ia_valid & ATTR_UID)
-		rc = ccs_chown_permission(dentry, NULL, attr->ia_uid, INVALID_GID);
+		rc = ccs_chown_permission(dentry, NULL, attr->ia_uid,
+					  INVALID_GID);
 	if (!rc && (attr->ia_valid & ATTR_GID))
-		rc = ccs_chown_permission(dentry, NULL, INVALID_UID, attr->ia_gid);
+		rc = ccs_chown_permission(dentry, NULL, INVALID_UID,
+					  attr->ia_gid);
 #else
 	if (attr->ia_valid & ATTR_UID)
 		rc = ccs_chown_permission(dentry, NULL, attr->ia_uid, -1);
@@ -902,6 +1030,26 @@ static int ccs_inode_setattr(struct dentry *dentry, struct iattr *attr)
 
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
+
+/**
+ * ccs_inode_getattr - Check permission for stat().
+ *
+ * @path: Pointer to "struct path".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_inode_getattr(const struct path *path)
+{
+	int rc = ccs_getattr_permission(path->mnt, path->dentry);
+	if (rc)
+		return rc;
+	while (!original_security_ops.inode_getattr);
+	return original_security_ops.inode_getattr(path);
+}
+
+#else
+
 /**
  * ccs_inode_getattr - Check permission for stat().
  *
@@ -919,6 +1067,8 @@ static int ccs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
 	return original_security_ops.inode_getattr(mnt, dentry);
 }
 
+#endif
+
 #if defined(CONFIG_SECURITY_PATH)
 
 #if defined(USE_UMODE_T)
@@ -1974,7 +2124,7 @@ static int ccs_sb_mount(char *dev_name, struct nameidata *nd, char *type,
 					      data_page);
 }
 
-#else
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
 
 /**
  * ccs_sb_pivotroot - Check permission for pivot_root().
@@ -2015,6 +2165,47 @@ static int ccs_sb_mount(char *dev_name, struct path *path, char *type,
 					      data_page);
 }
 
+#else
+
+/**
+ * ccs_sb_pivotroot - Check permission for pivot_root().
+ *
+ * @old_path: Pointer to "struct path".
+ * @new_path: Pointer to "struct path".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_sb_pivotroot(struct path *old_path, struct path *new_path)
+{
+	int rc = ccs_pivot_root_permission(old_path, new_path);
+	if (rc)
+		return rc;
+	while (!original_security_ops.sb_pivotroot);
+	return original_security_ops.sb_pivotroot(old_path, new_path);
+}
+
+/**
+ * ccs_sb_mount - Check permission for mount().
+ *
+ * @dev_name:  Name of device file.
+ * @path:      Pointer to "struct path".
+ * @type:      Name of filesystem type. Maybe NULL.
+ * @flags:     Mount options.
+ * @data_page: Optional data. Maybe NULL.
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int ccs_sb_mount(const char *dev_name, struct path *path,
+			const char *type, unsigned long flags, void *data_page)
+{
+	int rc = ccs_mount_permission(dev_name, path, type, flags, data_page);
+	if (rc)
+		return rc;
+	while (!original_security_ops.sb_mount);
+	return original_security_ops.sb_mount(dev_name, path, type, flags,
+					      data_page);
+}
+
 #endif
 
 /**
@@ -2170,608 +2361,6 @@ out:
 
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
-
-#include <linux/mount.h>
-#include <linux/fs_struct.h>
-
-/**
- * ccs_kernel_read - Wrapper for kernel_read().
- *
- * @file:   Pointer to "struct file".
- * @offset: Starting position.
- * @addr:   Buffer.
- * @count:  Size of @addr.
- *
- * Returns return value from kernel_read().
- */
-static int __init ccs_kernel_read(struct file *file, unsigned long offset,
-				  char *addr, unsigned long count)
-{
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
-	/*
-	 * I can't use kernel_read() because seq_read() returns -EPIPE
-	 * if &pos != &file->f_pos .
-	 */
-	mm_segment_t old_fs;
-	unsigned long pos = file->f_pos;
-	int result;
-	file->f_pos = offset;
-	old_fs = get_fs();
-	set_fs(get_ds());
-	result = vfs_read(file, (void __user *)addr, count, &file->f_pos);
-	set_fs(old_fs);
-	file->f_pos = pos;
-	return result;
-#else
-	return kernel_read(file, offset, addr, count);
-#endif
-}
-
-/**
- * ccs_find_symbol - Find function's address from /proc/kallsyms .
- *
- * @keyline: Function to find.
- *
- * Returns address of specified function on success, NULL otherwise.
- */
-static void *__init ccs_find_symbol(const char *keyline)
-{
-	struct file *file = NULL;
-	char *buf;
-	unsigned long entry = 0;
-	{
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
-		struct file_system_type *fstype = get_fs_type("proc");
-		struct vfsmount *mnt = vfs_kern_mount(fstype, 0, "proc", NULL);
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 8)
-		struct file_system_type *fstype = NULL;
-		struct vfsmount *mnt = do_kern_mount("proc", 0, "proc", NULL);
-#else
-		struct file_system_type *fstype = get_fs_type("proc");
-		struct vfsmount *mnt = kern_mount(fstype);
-#endif
-		struct dentry *root;
-		struct dentry *dentry;
-		/*
-		 * We embed put_filesystem() here because it is not exported.
-		 */
-		if (fstype)
-			module_put(fstype->owner);
-		if (IS_ERR(mnt))
-			goto out;
-		root = dget(mnt->mnt_root);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 16)
-		mutex_lock(&root->d_inode->i_mutex);
-		dentry = lookup_one_len("kallsyms", root, 8);
-		mutex_unlock(&root->d_inode->i_mutex);
-#else
-		down(&root->d_inode->i_sem);
-		dentry = lookup_one_len("kallsyms", root, 8);
-		up(&root->d_inode->i_sem);
-#endif
-		dput(root);
-		if (IS_ERR(dentry))
-			mntput(mnt);
-		else {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
-			struct path path = { mnt, dentry };
-			file = dentry_open(&path, O_RDONLY, current_cred());
-#else
-			file = dentry_open(dentry, mnt, O_RDONLY
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
-					   , current_cred()
-#endif
-					   );
-#endif
-		}
-	}
-	if (IS_ERR(file) || !file)
-		goto out;
-	buf = kmalloc(PAGE_SIZE, GFP_KERNEL);
-	if (buf) {
-		int len;
-		int offset = 0;
-		while ((len = ccs_kernel_read(file, offset, buf,
-					      PAGE_SIZE - 1)) > 0) {
-			char *cp;
-			buf[len] = '\0';
-			cp = strrchr(buf, '\n');
-			if (!cp)
-				break;
-			*(cp + 1) = '\0';
-			offset += strlen(buf);
-			cp = strstr(buf, keyline);
-			if (!cp)
-				continue;
-			*cp = '\0';
-			while (cp > buf && *(cp - 1) != '\n')
-				cp--;
-			entry = simple_strtoul(cp, NULL, 16);
-			break;
-		}
-		kfree(buf);
-	}
-	filp_close(file, NULL);
-out:
-	return (void *) entry;
-}
-
-#endif
-
-#if defined(CONFIG_ARM) && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-static int lsm_addr_calculator(struct file *file);
-static void * __init ccs_find_security_ops_on_arm(unsigned int *base);
-#endif
-
-#if defined(CONFIG_ARM) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
-/**
- * ccs_find_vfsmount_lock_on_arm - Find vfsmount_lock spinlock on ARM.
- *
- * @ip:   Address of dummy function's entry point.
- * @addr: Address of the variable which is used within @function.
- * @base: Address of function's entry point.
- *
- * Returns address of vfsmount_lock on success, NULL otherwise.
- */
-static void * __init ccs_find_vfsmount_lock_on_arm(unsigned int *ip,
-						   unsigned long addr,
-						   unsigned int *base)
-{
-	int i;
-	for (i = 0; i < 32; ip++, i++) {
-		static unsigned int *ip4ret;
-		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
-			continue;
-		ip = base + i;
-		ip4ret = (unsigned int *) (*(ip + 2 + ((*ip & 0xFFF) >> 2)));
-		return &ip4ret;
-	}
-	return NULL;
-}
-#endif
-
-/**
- * ccs_find_variable - Find variable's address using dummy.
- *
- * @function: Pointer to dummy function's entry point.
- * @addr:     Address of the variable which is used within @function.
- * @symbol:   Name of symbol to resolve.
- *
- * This trick depends on below assumptions.
- *
- * (1) @addr is found within 128 bytes from @function, even if additional
- *     code (e.g. debug symbols) is added.
- * (2) It is safe to read 128 bytes from @function.
- * (3) @addr != Byte code except @addr.
- */
-static void * __init ccs_find_variable(void *function, unsigned long addr,
-				       const char *symbol)
-{
-	int i;
-	u8 *base;
-	u8 *cp = function;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
-	if (*symbol == ' ')
-		base = ccs_find_symbol(symbol);
-	else
-#endif
-		base = __symbol_get(symbol);
-	if (!base)
-		return NULL;
-#if defined(CONFIG_ARM) && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-	if (function == lsm_addr_calculator)
-		return ccs_find_security_ops_on_arm((unsigned int *) base);
-#endif
-#if defined(CONFIG_ARM) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
-	return ccs_find_vfsmount_lock_on_arm(function, addr,
-					     (unsigned int *) base);
-#endif
-	/* First, assume absolute adressing mode is used. */
-	for (i = 0; i < 128; i++) {
-		if (*(unsigned long *) cp == addr)
-			return base + i;
-		cp++;
-	}
-	/* Next, assume PC-relative addressing mode is used. */
-	cp = function;
-	for (i = 0; i < 128; i++) {
-		if ((unsigned long) (cp + sizeof(int) + *(int *) cp) == addr) {
-			static void *cp4ret;
-			cp = base + i;
-			cp += sizeof(int) + *(int *) cp;
-			cp4ret = cp;
-			return &cp4ret;
-		}
-		cp++;
-	}
-	cp = function;
-	for (i = 0; i < 128; i++) {
-		if ((unsigned long) (long) (*(int *) cp) == addr) {
-			static void *cp4ret;
-			cp = base + i;
-			cp = (void *) (long) (*(int *) cp);
-			cp4ret = cp;
-			return &cp4ret;
-		}
-		cp++;
-	}
-	return NULL;
-}
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-
-/* Never mark this variable as __initdata . */
-static struct security_operations *ccs_security_ops;
-
-/**
- * lsm_addr_calculator - Dummy function which does identical to security_file_alloc() in security/security.c.
- *
- * @file: Pointer to "struct file".
- *
- * Returns return value from security_file_alloc().
- *
- * Never mark this function as __init in order to make sure that compiler
- * generates identical code for security_file_alloc() and this function.
- */
-static int lsm_addr_calculator(struct file *file)
-{
-	return ccs_security_ops->file_alloc_security(file);
-}
-
-#ifdef CONFIG_ARM
-/**
- * ccs_find_security_ops_on_arm - Find security_ops on ARM.
- *
- * @base: Address of security_file_alloc().
- *
- * Returns address of security_ops on success, NULL otherwise.
- */
-static void * __init ccs_find_security_ops_on_arm(unsigned int *base)
-{
-	static unsigned int *ip4ret;
-	int i;
-	const unsigned long addr = (unsigned long) &ccs_security_ops;
-	unsigned int *ip = (unsigned int *) lsm_addr_calculator;
-	for (i = 0; i < 32; ip++, i++) {
-		if (*(ip + 2 + ((*ip & 0xFFF) >> 2)) != addr)
-			continue;
-		ip = base + i;
-		ip4ret = (unsigned int *) (*(ip + 2 + ((*ip & 0xFFF) >> 2)));
-		return &ip4ret;
-	}
-	ip = (unsigned int *) lsm_addr_calculator;
-	for (i = 0; i < 32; ip++, i++) {
-		/*
-		 * Find
-		 *   ldr r3, [pc, #offset1]
-		 *   ldr r3, [r3, #offset2]
-		 * sequence.
-		 */
-		if ((*ip & 0xFFFFF000) != 0xE59F3000 ||
-		    (*(ip + 1) & 0xFFFFF000) != 0xE5933000)
-			continue;
-		ip4ret = (unsigned int *) (*(ip + 2 + ((*ip & 0xFFF) >> 2)));
-		ip4ret += (*(ip + 1) & 0xFFF) >> 2;
-		if ((unsigned long) ip4ret != addr)
-			continue;
-		ip = base + i;
-		ip4ret = (unsigned int *) (*(ip + 2 + ((*ip & 0xFFF) >> 2)));
-		ip4ret += (*(ip + 1) & 0xFFF) >> 2;
-		return &ip4ret;
-	}
-	return NULL;
-}
-#endif
-#endif
-
-/**
- * ccs_find_find_security_ops - Find address of "struct security_operations *security_ops".
- *
- * Returns pointer to "struct security_operations" on success, NULL otherwise.
- */
-static struct security_operations * __init ccs_find_security_ops(void)
-{
-	struct security_operations **ptr;
-	struct security_operations *ops;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-	void *cp;
-	/* Guess "struct security_operations *security_ops;". */
-	cp = ccs_find_variable(lsm_addr_calculator, (unsigned long)
-			       &ccs_security_ops, " security_file_alloc\n");
-	if (!cp) {
-		printk(KERN_ERR "Can't resolve security_file_alloc().\n");
-		goto out;
-	}
-	/* This should be "struct security_operations *security_ops;". */
-	ptr = *(struct security_operations ***) cp;
-#else
-	/* This is "struct security_operations *security_ops;". */
-	ptr = (struct security_operations **) __symbol_get("security_ops");
-#endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve security_ops structure.\n");
-		goto out;
-	}
-	printk(KERN_INFO "security_ops=%p\n", ptr);
-	ops = *ptr;
-	if (!ops) {
-		printk(KERN_ERR "No security_operations registered.\n");
-		goto out;
-	}
-	return ops;
-out:
-	return NULL;
-}
-
-/**
- * ccs_find_find_task_by_pid - Find address of find_task_by_pid().
- *
- * Returns true on success, false otherwise.
- */
-static bool __init ccs_find_find_task_by_pid(void)
-{
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-	void *ptr;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
-	ptr = ccs_find_symbol(" find_task_by_vpid\n");
-#else
-	ptr = __symbol_get("find_task_by_vpid");
-#endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_vpid().\n");
-		goto out;
-	}
-	ccsecurity_exports.find_task_by_vpid = ptr;
-	printk(KERN_INFO "find_task_by_vpid=%p\n", ptr);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
-	ptr = ccs_find_symbol(" find_task_by_pid_ns\n");
-#else
-	ptr = __symbol_get("find_task_by_pid_ns");
-#endif
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve find_task_by_pid_ns().\n");
-		goto out;
-	}
-	ccsecurity_exports.find_task_by_pid_ns = ptr;
-	printk(KERN_INFO "find_task_by_pid_ns=%p\n", ptr);
-	return true;
-out:
-	return false;
-#else
-	return true;
-#endif
-}
-
-#if defined(CONFIG_SMP) || defined(CONFIG_DEBUG_SPINLOCK) || LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
-
-/* Never mark this variable as __initdata . */
-static spinlock_t ccs_vfsmount_lock __cacheline_aligned_in_smp =
-SPIN_LOCK_UNLOCKED;
-
-static struct list_head *mount_hashtable;
-static int hash_mask, hash_bits;
-
-/**
- * hash - Copy of hash() in fs/namespace.c.
- *
- * @mnt: Pointer to "struct vfsmount".
- * @dentry: Pointer to "struct dentry".
- *
- * Returns hash value.
- */
-static inline unsigned long hash(struct vfsmount *mnt, struct dentry *dentry)
-{
-	unsigned long tmp = ((unsigned long) mnt / L1_CACHE_BYTES);
-	tmp += ((unsigned long) dentry / L1_CACHE_BYTES);
-	tmp = tmp + (tmp >> hash_bits);
-	return tmp & hash_mask;
-}
-
-/**
- * lsm_lu_mnt - Dummy function which does identical to lookup_mnt() in fs/namespace.c.
- *
- * @mnt:    Pointer to "struct vfsmount".
- * @dentry: Pointer to "struct dentry".
- *
- * Returns pointer to "struct vfsmount".
- *
- * Never mark this function as __init in order to make sure that compiler
- * generates identical code for lookup_mnt() and this function.
- */
-static struct vfsmount *lsm_lu_mnt(struct vfsmount *mnt, struct dentry *dentry)
-{
-	struct list_head *head = mount_hashtable + hash(mnt, dentry);
-	struct list_head *tmp = head;
-	struct vfsmount *p, *found = NULL;
-
-	spin_lock(&ccs_vfsmount_lock);
-	for (;;) {
-		tmp = tmp->next;
-		p = NULL;
-		if (tmp == head)
-			break;
-		p = list_entry(tmp, struct vfsmount, mnt_hash);
-		if (p->mnt_parent == mnt && p->mnt_mountpoint == dentry) {
-			found = mntget(p);
-			break;
-		}
-	}
-	spin_unlock(&ccs_vfsmount_lock);
-	return found;
-}
-
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
-
-/* Never mark this variable as __initdata . */
-static spinlock_t ccs_vfsmount_lock;
-
-/**
- * lsm_flwup - Dummy function which does identical to follow_up() in fs/namei.c.
- *
- * @mnt:    Pointer to "struct vfsmount *".
- * @dentry: Pointer to "struct dentry *".
- *
- * Returns 1 if followed up, 0 otehrwise.
- *
- * Never mark this function as __init in order to make sure that compiler
- * generates identical code for follow_up() and this function.
- */
-static int lsm_flwup(struct vfsmount **mnt, struct dentry **dentry)
-{
-	struct vfsmount *parent;
-	struct dentry *mountpoint;
-	spin_lock(&ccs_vfsmount_lock);
-	parent = (*mnt)->mnt_parent;
-	if (parent == *mnt) {
-		spin_unlock(&ccs_vfsmount_lock);
-		return 0;
-	}
-	mntget(parent);
-	mountpoint = dget((*mnt)->mnt_mountpoint);
-	spin_unlock(&ccs_vfsmount_lock);
-	dput(*dentry);
-	*dentry = mountpoint;
-	mntput(*mnt);
-	*mnt = parent;
-	return 1;
-}
-
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
-
-/* Never mark this variable as __initdata . */
-static spinlock_t ccs_vfsmount_lock;
-
-/**
- * lsm_pin - Dummy function which does identical to mnt_pin() in fs/namespace.c.
- *
- * @mnt: Pointer to "struct vfsmount".
- *
- * Returns nothing.
- *
- * Never mark this function as __init in order to make sure that compiler
- * generates identical code for mnt_pin() and this function.
- */
-static void lsm_pin(struct vfsmount *mnt)
-{
-	spin_lock(&ccs_vfsmount_lock);
-	mnt->mnt_pinned++;
-	spin_unlock(&ccs_vfsmount_lock);
-}
-
-#endif
-
-/**
- * ccs_find_vfsmount_lock - Find address of "spinlock_t vfsmount_lock".
- *
- * Returns true on success, false otherwise.
- */
-static bool __init ccs_find_vfsmount_lock(void)
-{
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 3)
-	void *cp;
-	spinlock_t *ptr;
-	/* Guess "spinlock_t vfsmount_lock;". */
-	cp = ccs_find_variable(lsm_lu_mnt, (unsigned long) &ccs_vfsmount_lock,
-			       " lookup_mnt\n");
-	if (!cp) {
-		printk(KERN_ERR "Can't resolve lookup_mnt().\n");
-		goto out;
-	}
-	/* This should be "spinlock_t *vfsmount_lock;". */
-	ptr = *(spinlock_t **) cp;
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve vfsmount_lock .\n");
-		goto out;
-	}
-	ccsecurity_exports.vfsmount_lock = ptr;
-	printk(KERN_INFO "vfsmount_lock=%p\n", ptr);
-	return true;
-out:
-	return false;
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 15)
-	void *cp;
-	spinlock_t *ptr;
-	/* Guess "spinlock_t vfsmount_lock;". */
-	cp = ccs_find_variable(lsm_flwup, (unsigned long) &ccs_vfsmount_lock,
-			       "follow_up");
-	if (!cp) {
-		printk(KERN_ERR "Can't resolve follow_up().\n");
-		goto out;
-	}
-	/* This should be "spinlock_t *vfsmount_lock;". */
-	ptr = *(spinlock_t **) cp;
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve vfsmount_lock .\n");
-		goto out;
-	}
-	ccsecurity_exports.vfsmount_lock = ptr;
-	printk(KERN_INFO "vfsmount_lock=%p\n", ptr);
-	return true;
-out:
-	return false;
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
-	void *cp;
-	spinlock_t *ptr;
-	/* Guess "spinlock_t vfsmount_lock;". */
-	cp = ccs_find_variable(lsm_pin, (unsigned long) &ccs_vfsmount_lock,
-			       "mnt_pin");
-	if (!cp) {
-		printk(KERN_ERR "Can't resolve mnt_pin().\n");
-		goto out;
-	}
-	/* This should be "spinlock_t *vfsmount_lock;". */
-	ptr = *(spinlock_t **) cp;
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve vfsmount_lock .\n");
-		goto out;
-	}
-	ccsecurity_exports.vfsmount_lock = ptr;
-	printk(KERN_INFO "vfsmount_lock=%p\n", ptr);
-	return true;
-out:
-	return false;
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
-	void *ptr = ccs_find_symbol(" __d_path\n");
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve __d_path().\n");
-		return false;
-	}
-	ccsecurity_exports.__d_path = ptr;
-	printk(KERN_INFO "__d_path=%p\n", ptr);
-	return true;
-#else
-	void *ptr = ccs_find_symbol(" d_absolute_path\n");
-	if (!ptr) {
-		printk(KERN_ERR "Can't resolve d_absolute_path().\n");
-		return false;
-	}
-	ccsecurity_exports.d_absolute_path = ptr;
-	printk(KERN_INFO "d_absolute_path=%p\n", ptr);
-	return true;
-#endif
-}
-
-#else
-
-/* Never mark this variable as __initdata . */
-static spinlock_t ccs_vfsmount_lock;
-
-/**
- * ccs_find_vfsmount_lock - Find address of "spinlock_t vfsmount_lock".
- *
- * Returns true on success, false otherwise.
- */
-static bool __init ccs_find_vfsmount_lock(void)
-{
-	ccsecurity_exports.vfsmount_lock = &ccs_vfsmount_lock;
-	return true;
-}
-
-#endif
-
 /*
  * Why not to copy all operations by "original_security_ops = *ops" ?
  * Because copying byte array is not atomic. Reader checks
@@ -2802,6 +2391,7 @@ static void __init ccs_update_security_ops(struct security_operations *ops)
 #else
 	swap_security_ops(task_alloc_security);
 	swap_security_ops(task_free_security);
+	swap_security_ops(bprm_alloc_security);
 	swap_security_ops(bprm_free_security);
 #endif
 	/* Security context updater for successful execve(). */
@@ -2880,10 +2470,30 @@ static void __init ccs_update_security_ops(struct security_operations *ops)
  */
 static int __init ccs_init(void)
 {
-	struct security_operations *ops = ccs_find_security_ops();
-	if (!ops || !ccs_find_find_task_by_pid() ||
-	    !ccs_find_vfsmount_lock())
-		return -EINVAL;
+	struct security_operations *ops = probe_security_ops();
+	if (!ops)
+		goto out;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+	ccsecurity_exports.find_task_by_vpid = probe_find_task_by_vpid();
+	if (!ccsecurity_exports.find_task_by_vpid)
+		goto out;
+	ccsecurity_exports.find_task_by_pid_ns = probe_find_task_by_pid_ns();
+	if (!ccsecurity_exports.find_task_by_pid_ns)
+		goto out;
+#endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
+	ccsecurity_exports.vfsmount_lock = probe_vfsmount_lock();
+	if (!ccsecurity_exports.vfsmount_lock)
+		goto out;
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
+	ccsecurity_exports.__d_path = probe___d_path();
+	if (!ccsecurity_exports.__d_path)
+		goto out;
+#else
+	ccsecurity_exports.d_absolute_path = probe_d_absolute_path();
+	if (!ccsecurity_exports.d_absolute_path)
+		goto out;
+#endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 	{
 		int idx;
@@ -2895,10 +2505,12 @@ static int __init ccs_init(void)
 #endif
 	ccs_main_init();
 	ccs_update_security_ops(ops);
-	printk(KERN_INFO "AKARI: 1.0.27   2012/05/05\n");
+	printk(KERN_INFO "AKARI: 1.0.32   2015/04/08\n");
 	printk(KERN_INFO
 	       "Access Keeping And Regulating Instrument registered.\n");
 	return 0;
+out:
+	return -EINVAL;
 }
 
 module_init(ccs_init);
@@ -2976,6 +2588,36 @@ struct ccs_security *ccs_find_task_security(const struct task_struct *task)
 		if (ptr->pid != task->pids[PIDTYPE_PID].pid)
 			continue;
 		rcu_read_unlock();
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 31)
+		/*
+		 * Current thread needs to transit from old domain to new
+		 * domain before do_execve() succeeds in order to check
+		 * permission for interpreters and environment variables using
+		 * new domain's ACL rules. The domain transition has to be
+		 * visible from other CPU in order to allow interactive
+		 * enforcing mode. Also, the domain transition has to be
+		 * reverted if do_execve() failed. However, an LSM hook for
+		 * reverting domain transition is missing.
+		 *
+		 * security_prepare_creds() is called from prepare_creds() from
+		 * prepare_bprm_creds() from do_execve() before setting
+		 * current->in_execve flag, and current->in_execve flag is
+		 * cleared by the time next do_execve() request starts.
+		 * This means that we can emulate the missing LSM hook for
+		 * reverting domain transition, by calling this function from
+		 * security_prepare_creds(). 
+		 *
+		 * If current->in_execve is not set but ptr->ccs_flags has
+		 * CCS_TASK_IS_IN_EXECVE set, it indicates that do_execve()
+		 * has failed and reverting domain transition is needed.
+		 */
+		if (task == current &&
+		    (ptr->ccs_flags & CCS_TASK_IS_IN_EXECVE) &&
+		    !current->in_execve) {
+			ccs_debug_trace("4");
+			ccs_clear_execve(-1, ptr);
+		}
+#else
 		/*
 		 * Current thread needs to transit from old domain to new
 		 * domain before do_execve() succeeds in order to check
@@ -3008,17 +2650,10 @@ struct ccs_security *ccs_find_task_security(const struct task_struct *task)
 		 */
 		if (task == current && ptr->cred &&
 		    atomic_read(&ptr->cred->usage) == 1) {
-#ifdef CONFIG_AKARI_DEBUG
-			static bool done;
-			if (!done) {
-				printk(KERN_INFO "AKARI: Reverting domain "
-				       "transition because do_execve() has "
-				       "failed.\n");
-				done = true;
-			}
-#endif
+			ccs_debug_trace("4");
 			ccs_clear_execve(-1, ptr);
 		}
+#endif
 		return ptr;
 	}
 	rcu_read_unlock();
@@ -3112,14 +2747,14 @@ static struct ccs_security *ccs_find_cred_security(const struct cred *cred)
  *
  * Returns nothing.
  *
- * Since security_task_free_security() is missing, I can't release memory
- * associated with "struct task_struct" when a task dies. Therefore, I hold
- * a reference on "struct pid" and runs garbage collection when associated
+ * Since security_task_free() is missing, I can't release memory associated
+ * with "struct task_struct" when a task dies. Therefore, I hold a reference on
+ * "struct pid" and runs garbage collection when associated
  * "struct task_struct" has gone.
  */
 static void ccs_task_security_gc(void)
 {
-	static DEFINE_MUTEX(lock);
+	static DEFINE_SPINLOCK(lock);
 	static atomic_t gc_counter = ATOMIC_INIT(0);
 	unsigned int idx;
 	/*
@@ -3133,9 +2768,9 @@ static void ccs_task_security_gc(void)
 	if (!atomic_read(&ccs_in_execve_tasks) &&
 	    atomic_inc_return(&gc_counter) < 1024)
 		return;
-	atomic_set(&gc_counter, 0);
-	if (mutex_lock_interruptible(&lock))
+	if (!spin_trylock(&lock))
 		return;
+	atomic_set(&gc_counter, 0);
 	rcu_read_lock();
 	for (idx = 0; idx < CCS_MAX_TASK_SECURITY_HASH; idx++) {
 		struct ccs_security *ptr;
@@ -3147,7 +2782,7 @@ static void ccs_task_security_gc(void)
 		}
 	}
 	rcu_read_unlock();
-	mutex_unlock(&lock);
+	spin_unlock(&lock);
 }
 
 #endif
